from zemfrog.decorators import is_json_request, json_renderer
from zemfrog.helper import db_add, db_delete, db_update
from flask import request
from extensions.sqlalchemy import db
from ${src_model} import ${name}
from ${src_schema} import ${name}Schema

schema = ${name}Schema()

@json_renderer
def get():
    """
    Read all data.
    """

    data = ${name}.query.all()
    return schema.dump(data, many=True)

@is_json_request
@json_renderer
def add():
    """
    Add data.
    """

    json = request.get_json()
    found = ${name}.query.filter_by(**json).first()
    if not found:
        model = schema.load(json)
        db_add(db, model)
        status_code = 200
        reason = "Data berhasil ditambahkan."

    else:
        status_code = 403
        reason = "Data sudah ditemukan."

    return {
        "status_code": status_code,
        "reason": reason
    }

@is_json_request
@json_renderer
def update():
    """
    Update data.
    """

    json = request.get_json()
    new_data = json.pop("__update__", None)
    if new_data and isinstance(new_data, dict):
        model = ${name}.query.filter_by(**json).first()
        if not model:
            status_code = 404
            reason = "Data tidak ditemukan."

        else:
            db_update(db, model, **new_data)
            status_code = 200
            reason = "Data berhasil diperbaharui."

    else:
        status_code = 403
        reason = "Data baru tidak ditemukan."

    return {
        "status_code": status_code,
        "reason": reason
    }

@is_json_request
@json_renderer
def delete():
    """
    Delete data.
    """

    json = request.get_json()
    model = ${name}.query.filter_by(**json).first()
    if model:
        db_delete(db, model)
        status_code = 200
        reason = "Data berhasil dihapus."

    else:
        status_code = 404
        reason = "Data tidak ditemukan."

    return {
        "status_code": status_code,
        "reason": reason
    }

endpoint = "${url_prefix}"
url_prefix = "/${url_prefix}"
routes = [
    ("/get", get, ["GET"]),
    ("/add", add, ["POST"]),
    ("/update", update, ["PUT"]),
    ("/delete", delete, ["DELETE"])
]



import click
from flask.cli import with_appcontext
from ..generator import g_api, g_api_crud

@click.group("api")
def group():
    """
    API generator.
    """

@group.command()
@with_appcontext
@click.argument("name")
@click.option("--crud", is_flag=True, help="Membuat crud berdasarkan ORM model.")
def new(name, crud):
    """
    Create api ler.
    """

    func = g_api
    if crud:
        func = g_api_crud
    func(name)

command = group



from zemfrog.decorators import json_renderer

@json_renderer
def get():
    return {
        "text": "welcome :')"
    }


endpoint = "${url_prefix}"
url_prefix = "/${url_prefix}"
routes = [
    ("/get", get, ["GET"])
]



import os
from flask import Flask
from os import getenv
from glob import glob
from importlib import import_module

from flask.blueprints import Blueprint

from .generator import g_schema
from .exception import ZemfrogEnvironment
from .helper import import_attr, search_model

def load_config(app: Flask):
    env = getenv("ZEMFROG_ENV")
    if not env:
        raise ZemfrogEnvironment("environment not found")

    app.config.from_object("config." + env.capitalize())

def load_extensions(app: Flask):
    extensions = app.config.get("EXTENSIONS", [])
    for ext in extensions:
        ext = import_module(ext)
        init_func = getattr(ext, "init_app")
        init_func(app)

def load_models(app: Flask):
    app.models = {}
    true = app.config.get("CREATE_DB")
    if true:
        models = [x.rsplit(".", 1)[0].replace(os.sep, ".") for x in glob("models/**/*.py", recursive=True)]
        for m in models:
            if "__init__" in m:
                m = m.replace(".__init__", "")
            mod = import_module(m)
            app.models[mod] = m

        app.extensions["sqlalchemy"].db.create_all()

def load_commands(app: Flask):
    commands = app.config.get("COMMANDS", [])
    for cmd in commands:
        cmd = cmd + ".command"
        cmd = import_attr(cmd)
        app.cli.add_command(cmd)

def load_blueprints(app: Flask):
    blueprints = app.config.get("BLUEPRINTS", [])
    for bp in blueprints:
        bp = bp + ".routes.blueprint"
        bp = import_attr(bp)
        app.register_blueprint(bp)

def load_apis(app: Flask):
    apis = app.config.get("APIS", [])
    api: Blueprint = import_attr("api.api")
    for res in apis:
        res = import_module(res)
        endpoint = res.endpoint
        url_prefix = res.url_prefix
        routes = res.routes
        for detail in routes:
            route, view, methods = detail
            url = url_prefix + route
            e = endpoint + "_" + view.__name__
            api.add_url_rule(url, e, view_func=view, methods=methods)

    app.register_blueprint(api)

def load_services(app: Flask):
    services = app.config.get("SERVICES", [])
    for sv in services:
        import_module(sv)

def load_schemas(app: Flask):
    sql = app.extensions["sqlalchemy"]
    tables = sql.db.metadata.tables.keys()
    for t in tables:
        t = t.split("_")
        t = "".join([x.capitalize() for x in t])
        src = search_model(t)
        g_schema(t, src)



from importlib import import_module
import os
import string
from flask import current_app

from .helper import copy_template, search_model

def g_project(name):
    print("Creating %r project..." % name)
    copy_template("project", name)

def g_api(name):
    print("Creating rest api %r... " % name, end="")
    copy_template("api", "api")
    old_filename = os.path.join("api", "name.py")
    with open(old_filename) as fp:
        old_data = fp.read()
        py_t = string.Template(old_data)
        new_data = py_t.safe_substitute(name=name, url_prefix=name.lower())

    os.remove(old_filename)
    new_filename = os.path.join("api", name.lower() + ".py")
    with open(new_filename, "w") as fp:
        fp.write(new_data)

    print("(done)")

def g_api_crud(name):
    src_model = search_model(name)
    if not src_model:
        print("Model not found %r" % name)
        return

    src_schema = src_model.replace("models", "schema")
    print("Creating rest api (crud) %r... " % name, end="")
    copy_template("crud", "api")
    old_filename = os.path.join("api", "name.py")
    with open(old_filename) as fp:
        old_data = fp.read()
        py_t = string.Template(old_data)
        new_data = py_t.safe_substitute(name=name, url_prefix=name.lower(), src_model=src_model, src_schema=src_schema)

    os.remove(old_filename)
    new_filename = os.path.join("api", name.lower() + ".py")
    with open(new_filename, "w") as fp:
        fp.write(new_data)

    print("(done)")

def g_blueprint(name):
    print("Creating blueprint %r... " % name, end="")
    copy_template("blueprint", name.lower())
    filename = os.path.join(name.lower(), "routes.py")
    with open(filename) as fp:
        old_data = fp.read()
        py_t = string.Template(old_data)
        new_data = py_t.safe_substitute(name=name)

    with open(filename, "w") as fp:
        fp.write(new_data)

    print("(done)")

def g_schema(name, src):
    print("Creating model schema %r... " % name, end="")
    copy_template("schema", "schema")
    old_filename = os.path.join("schema", "name.py")
    with open(old_filename) as fp:
        old_data = fp.read()
        py_t = string.Template(old_data)
        new_data = py_t.safe_substitute(name=name, src_model=src)

    os.remove(old_filename)
    srcfile = import_module(src).__file__.replace("models" + os.sep, "schema" + os.sep)
    dirname = os.path.dirname(srcfile).replace("models" + os.sep, "schema" + os.sep)
    try:
        os.makedirs(dirname)
    except FileExistsError:
        pass

    with open(srcfile, "w") as fp:
        fp.write(new_data)

    print("(done)")



import click

from .generator import g_project

@click.group()
def main():
    """
    Zemfrog command line 
    """

@main.command("create")
@click.argument("name")
def create(name):
    """
    Create a project.
    """

    g_project(name)



import os

from importlib import import_module
from distutils.dir_util import copy_tree
from flask import current_app
from flask_sqlalchemy import Model

from .exception import ZemfrogTemplateNotFound

TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), "templates")

def get_template(name):
    t = os.path.join(TEMPLATE_DIR, name)
    if not os.path.isdir(t):
        raise ZemfrogTemplateNotFound("unknown template %r" % name)

    return t

def copy_template(name, dst):
    t = get_template(name)
    copy_tree(t, dst)

def import_attr(module):
    pkg, name = module.rsplit(".", 1)
    mod = import_module(pkg)
    return getattr(mod, name)

def search_model(name):
    for mod in current_app.models:
        childs = dir(mod)
        for c in childs:
            c = getattr(mod, c)
            try:
                if issubclass(c, Model):
                    tbl_name = c.__name__
                    if tbl_name == name:
                        src = current_app.models.get(mod)
                        return src
            except TypeError:
                pass

def db_add(db, model):
    db.session.add(model)
    db.session.commit()

def db_delete(db, model):
    db.session.delete(model)
    db.session.commit()

def db_update(db, model, **kwds):
    for k, v in kwds.items():
        setattr(model, k, v)
    db.session.commit()



from flask import request, jsonify
from functools import wraps

def is_json_request(func):
    @wraps(func)
    def wrapper(*args, **kwds):
        json = request.json
        if not json and request.method != "GET":
            return jsonify(status_code=403, reason="Bukan json request!")
        return func(*args, **kwds)
    return wrapper

def json_renderer(func):
    @wraps(func)
    def wrapper(*args, **kwds):
        result = func(*args, **kwds)
        return jsonify(result)
    return wrapper



import click

from ..generator import g_blueprint

@click.group("blueprint")
def group():
    """
    Blueprint generator.
    """

@group.command()
@click.argument("name")
def new(name):
    """
    Create blueprint ler.
    """

    g_blueprint(name)

command = group



from flask import Flask
from . import loader
from celery import Celery

def make_celery(app: Flask):
    celery = Celery(
        app.import_name,
        backend=app.config['CELERY_RESULT_BACKEND'],
        broker=app.config['CELERY_BROKER_URL']
    )
    celery.conf.update(app.config)

    class ContextTask(celery.Task):
        def __call__(self, *args, **kwargs):
            with app.app_context():
                return self.run(*args, **kwargs)

    celery.Task = ContextTask
    return celery

def create_app(name):
    app = Flask(name)

    with app.app_context():
        loader.load_config(app)
        loader.load_extensions(app)
        loader.load_models(app)
        loader.load_blueprints(app)
        loader.load_apis(app)
        loader.load_commands(app)
        loader.load_services(app)
        loader.load_schemas(app)

    return app



class Development(object):
    SECRET_KEY = "Your secret key!"
    SQLALCHEMY_DATABASE_URI = "sqlite:///db.sqlite"
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    DEBUG = True
    EXTENSIONS = [
        "extensions.sqlalchemy",
        "extensions.marshmallow",
        "extensions.migrate"
    ]
    COMMANDS = [
        "zemfrog.commands.api",
        "zemfrog.commands.blueprint"
    ]
    BLUEPRINTS = []
    APIS = []
    CREATE_DB = True
    CELERY_RESULT_BACKEND = None
    CELERY_BROKER_URL = None

class Production(Development):
    DEBUG = False

class Testing(Development):
    TESTING = True
